/*
     grouplist.c -- determine list of all groups in the spool.
     Copyright (C) 2002  Matthias Andree

     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
     License as published by the Free Software Foundation.

     This library is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
     Lesser General Public License for more details.

     You should have received a copy of the GNU Lesser General Public
     License along with this library; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "leafnode.h"
#include "mastring.h"

#include <unistd.h>
#include <sys/types.h>
#include <dirent.h>
#include <sys/stat.h>
#include <string.h>
#include <stdlib.h>
#include <assert.h>

#ifdef TEST
const char *spooldir;
#endif

static int myscandir(struct stringlist **l, const char *prefix)
{
    DIR *d = opendir(".");
    struct dirent *de;
    int added = 0, founddir = 0;

    assert(l);
    assert(prefix);

    if (d == NULL) return -1;
    while ((de = readdir(d)) != NULL) {
	struct stat st;
	mastr *p = mastr_new(4096);

	if (strcmp(de->d_name, ".overview") != 0
	    && strchr(de->d_name, '.') != NULL) {
	    mastr_delete(p);
	    continue;
	}
	if (lstat(de->d_name, &st)) goto bail;
	if (added == 0 && S_ISREG(st.st_mode)) {
	    added = 1;
	    prependtolist(l, prefix);
	} else if (S_ISDIR(st.st_mode)) {
	    founddir = 1;
	    if (prefix != NULL && *prefix != '\0')
		(void)mastr_vcat(p, prefix, ".", de->d_name, NULL);
	    else
		(void)mastr_cpy(p, de->d_name);
	    if (chdir(de->d_name)) goto bail;
	    if (myscandir(l, mastr_str(p))) goto bail;
	    if (chdir("..")) goto bail;
	    mastr_delete(p);
	    continue;
	bail:
	    mastr_delete(p);
	    (void)closedir(d);
	    return -1;
	}
	mastr_delete(p);
    }
    (void)closedir(d);
    if (!added && !founddir) {
	prependtolist(l, prefix);
    }
    return 0;
}

/*@null@*/ struct stringlist *
get_grouplist(void)
{
    struct stringlist *g = NULL;

    if (chdir(spooldir)) return NULL;
    if (myscandir(&g, "") != 0 && g != NULL) {
	freelist(g);
	g = NULL;
    }
    return g;
}

#ifdef TEST
int verbose=0;
int debug=0;

int main(int argc, char **argv) {
    struct stringlist *t, *q, *l;

    if (argc > 1) { spooldir = argv[1]; }
    l = get_grouplist();
    if (!l) fputs("failed\n", stderr);
    else for (t = l; t; ) {
	puts(t->string);
	q = t->next;
	free(t);
	t = q;
    }
    return 0;
}
#endif
